@@ -1,4 +1,4 @@
-/* Copyright (C) 2019-2023 Free Software Foundation, Inc.
+/* Copyright (C) 2019-2024 Free Software Foundation, Inc.
 
    This file is part of LIBF7, which is part of GCC.
 
@@ -352,7 +352,7 @@ float f7_get_float (const f7_t *aa)
 
   return make_float (mant);
 }
-F7_PURE ALIAS (f7_get_float, f7_truncdfsf2)
+ALIAS (f7_get_float, f7_truncdfsf2)
 #endif // F7MOD_get_float_
 
 #define DBL_DIG_EXP   11
@@ -572,7 +572,7 @@ int32_t f7_get_s32 (const f7_t *aa)
   extern int32_t to_s32 (const f7_t*, uint8_t) F7ASM(f7_to_integer_asm);
   return to_s32 (aa, 0x1f);
 }
-F7_PURE ALIAS (f7_get_s32, f7_fixdfsi)
+ALIAS (f7_get_s32, f7_fixdfsi)
 #endif // F7MOD_get_s32_
 
 
@@ -583,7 +583,7 @@ F7_PURE ALIAS (f7_get_s32, f7_fixdfsi)
   extern int64_t to_s64 (const f7_t*, uint8_t) F7ASM(f7_to_integer_asm);
   return to_s64 (aa, 0x3f);
 }
-F7_PURE ALIAS (f7_get_s64, f7_fixdfdi)
+ALIAS (f7_get_s64, f7_fixdfdi)
 #endif // F7MOD_get_s64_
 
 #ifdef F7MOD_get_u16_
@@ -603,7 +603,7 @@ uint32_t f7_get_u32 (const f7_t *aa)
   extern uint32_t to_u32 (const f7_t*, uint8_t) F7ASM(f7_to_unsigned_asm);
   return to_u32 (aa, 0x1f);
 }
-F7_PURE ALIAS (f7_get_u32, f7_fixunsdfsi)
+ALIAS (f7_get_u32, f7_fixunsdfsi)
 #endif // F7MOD_get_u32_
 
 
@@ -614,7 +614,7 @@ uint64_t f7_get_u64 (const f7_t *aa)
   extern int64_t to_u64 (const f7_t*, uint8_t) F7ASM(f7_to_unsigned_asm);
   return to_u64 (aa, 0x3f);
 }
-F7_PURE ALIAS (f7_get_u64, f7_fixunsdfdi)
+ALIAS (f7_get_u64, f7_fixunsdfdi)
 #endif // F7MOD_get_u64_
 
 
@@ -1493,6 +1493,9 @@ void f7_horner (f7_t *cc, const f7_t *xx, uint8_t n_coeff, const f7_t *coeff,
 
   f7_copy_flash (yy, pcoeff);
 
+  if (yy->flags & F7_FLAG_plusx)
+    f7_Iadd (yy, xx);
+
   while (1)
     {
       --pcoeff;
@@ -1636,10 +1639,10 @@ void f7_exp (f7_t *cc, const f7_t *aa)
     return f7_set_nan (cc);
 
   /* The maximal exponent of 2 for a double is 1023, hence we may limit
-     to  |A| < 1023 * ln2 ~ 709.  We limit to  1024 = 2^10  */
+     to  |A| < 1023 * ln2 ~ 709.  We limit to  1024 ~ 1.99 * 2^9  */
 
   if (f7_class_inf (a_class)
-      || (f7_class_nonzero (a_class) && aa->expo >= 10))
+      || (f7_class_nonzero (a_class) && aa->expo >= 9))
     {
       if (f7_class_sign (a_class))
 	return f7_clr (cc);
@@ -1749,33 +1752,20 @@ void f7_powi (f7_t *cc, const f7_t *aa, int ii)
 {
   uint16_t u16 = ii;
   f7_t xx27, *xx2 = &xx27;
-  bool cc_is_one = true;
-  bool expo_is_neg = false;
 
   if (ii < 0)
-    {
-      u16 = -u16;
-      expo_is_neg = true;
-    }
+    u16 = -u16;
 
   f7_copy (xx2, aa);
 
+  f7_set_u16 (cc, 1);
+
   while (1)
     {
       if (u16 & 1)
-	{
-	  if (cc_is_one)
-	    {
-	      // C *= X2 simplifies to C = X2.
-	      f7_copy (cc, xx2);
-	      cc_is_one = false;
-	    }
-	  else
-	    f7_Imul (cc, xx2);
-	}
+	f7_Imul (cc, xx2);
 
-      if (! cc_is_one
-	  && ! f7_is_nonzero (cc))
+      if (! f7_is_nonzero (cc))
 	break;
 
       u16 >>= 1;
@@ -1784,10 +1774,8 @@ void f7_powi (f7_t *cc, const f7_t *aa, int ii)
       f7_Isquare (xx2);
     }
 
-  if (cc_is_one)
-    f7_set_u16 (cc, 1);
-  else if (expo_is_neg)
-    f7_div1 (cc, cc);
+  if (ii < 0)
+    f7_div1 (xx2, aa);
 }
 #endif // F7MOD_powi_
 
